<?php

defined('ABSPATH') or die('Plugin file cannot be accessed directly.');

class EngagebayGFAddonAPI
{
    private array $headers;

    public function __construct()
    {
        $this->headers = array(
            'Content-type' => 'application/json',
            'Accept' => 'application/json',
            'ebwhitelist' => true
        );
    }

    private function setAuthorizationHeader(string $value)
    {
        $this->headers['Authorization'] = $value;
    }

    public function get_domain_info()
    {
        $rest_api_key = EngageBayGFAddon::eb_gf_rest_api_key();
        if (empty($rest_api_key)) {
            return null;
        }

        $this->setAuthorizationHeader($rest_api_key);

        $args = array(
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
        );

        $request = wp_remote_get(EngaegBayGFAddonVars::GET_DOMAIN_INFO, $args);
        $result = wp_remote_retrieve_body($request);

        return json_decode($result, true);
    }

    public function get_js_api_key(string $rest_api_key)
    {
        $this->setAuthorizationHeader($rest_api_key);

        $args = array(
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
        );

        $request = wp_remote_get(EngaegBayGFAddonVars::GET_JS_API_KEY, $args);

        return wp_remote_retrieve_body($request);
    }

    public function get_custom_fields()
    {
        $this->setAuthorizationHeader(EngageBayGFAddon::eb_gf_rest_api_key());

        $args = array(
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
        );

        $request = wp_remote_get(EngaegBayGFAddonVars::GET_CUSTOM_FIELDS, $args);
        $customFields = wp_remote_retrieve_body($request);

        if ($customFields) {
            return json_decode($customFields, true);
        }

        return false;
    }

    public function is_valid_rest_api_key(string $rest_api_key)
    {
        $this->headers['Content-type'] = 'application/x-www-form-urlencoded';

        $args_post = array(
            'method' => 'POST',
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
            'body' => array('apiKey' => $rest_api_key),
        );

        $response = wp_remote_post(EngaegBayGFAddonVars::VALIDATE_REST_API_KEY, $args_post);
        return !($response['response']['code'] == 401);

    }

    public function send_ecommerce_hook()
    {
        $email = $this->eb_gf_get_current_user_email();

        if (!$email) {
            return;
        }

        $payload = array(
            'eventName' => 'UNINSTALL',
            'payload' => array(
                'url' => home_url(),
                'email' => $email,
            )
        );

        $args_post = array(
            'method' => 'POST',
            'timeout' => 120,
            'sslverify' => false,
            'headers' => array(
                'Content-type' => 'application/json',
                'Accept' => 'application/json',
                'ebwhitelist' => true
            ),
            'body' => json_encode($payload),
        );

        wp_remote_post(EngaegBayGFAddonVars::ECOMMERCE_HOOK, $args_post);
    }

    public function search_contact(string $email)
    {
        $rest_api_key = EngageBayGFAddon::eb_gf_rest_api_key();
        if (empty($rest_api_key)) {
            return null;
        }

        $this->setAuthorizationHeader($rest_api_key);

        $args = array(
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
        );

        $request = wp_remote_get(sprintf(EngaegBayGFAddonVars::SEARCH_CONTACT, $email), $args);
        $result = wp_remote_retrieve_body($request);

        return json_decode($result, true);
    }

    public function add_contact(array $contact_data)
    {
        $rest_api_key = EngageBayGFAddon::eb_gf_rest_api_key();
        if (empty($rest_api_key)) {
            return null;
        }

        $this->setAuthorizationHeader($rest_api_key);

        $args_post = array(
            'method' => 'POST',
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
            'body' => json_encode($contact_data)
        );

        wp_remote_post(EngaegBayGFAddonVars::ADD_CONTACT, $args_post);
    }

    public function update_contact(array $contact_data)
    {
        $rest_api_key = EngageBayGFAddon::eb_gf_rest_api_key();
        if (empty($rest_api_key)) {
            return null;
        }

        $this->setAuthorizationHeader($rest_api_key);

        $args_post = array(
            'method' => 'PUT',
            'timeout' => 120,
            'sslverify' => false,
            'headers' => $this->headers,
            'body' => json_encode($contact_data)
        );

        wp_remote_post(EngaegBayGFAddonVars::UPDATE_CONTACT, $args_post);
    }

    private function eb_gf_get_current_user_email()
    {
        // Get the current user's ID
        $user_id = get_current_user_id();

        // Get the user data
        $user = get_user_by('ID', $user_id);

        // Check if the user data exists
        if ($user) {
            return $user->user_email;
        }

        return null;
    }
}