<?php

class EngageBayGFAddonFormHandler
{
    public function eb_gf_add_actions()
    {
        add_action('gform_after_submission', array($this, 'eb_gf_on_submit_form'), 10, 2);
    }

    public function eb_gf_settings($form)
    {
        return array(
            array(
                'title' => __('Gravity Form Field Mapping with EngageBay Fields', 'engagebaygfaddon'),
                'fields' => array(
                    array(
                        'name' => 'mapGravityFormsToEngagebay',
                        'label' => __('Configuration', 'engagebaygfaddon'),
                        'onclick' => "jQuery(this).parents('form').submit();",
                        'type' => 'checkbox',
                        'choices' => array(
                            array(
                                'label' => sprintf(__('Integrate %s with EngageBay', 'engagebaygfaddon'), $form['title']),
                                'name' => 'engagebayMapEnabled',
                            ),
                        ),
                    ),
                ),
            ),
            array(
                'title' => __('Fields Mapping', 'engagebaygfaddon'),
                'name' => 'formConnections',
                'dependency' => array('field' => 'engagebayMapEnabled', 'values' => array(1)),
                'fields' => array(
                    array(
                        'name' => 'fieldMap',
                        'label' => __('Map EngageBay Fields with Gravity Forms', 'engagebaygfaddon'),
                        'type' => 'field_map',
                        'field_map' => $this->eb_gf_engagebay_fields(),
                        'tooltip' => '<h6>' . __('Field Mapping', 'engagebaygfaddon') . '</h6>' . __('Map EngageBay Fields with Gravity Form Fields', 'engagebaygfaddon'),
                    ),
                    array(
                        'name' => 'tag',
                        'label' => __('Add Tag', 'engagebaygfaddon'),
                        'type' => 'text',
                        'tooltip' => '<h6>' . __('Add Tag', 'engagebaygfaddon') . '</h6>' . __('Add a tag to identify the contact in EngageBay which are coming from the Gravity forms', 'engagebaygfaddon'),
                    ),
                ),
            ),
            array(
                'name' => 'formConnections',
                'fields' => array(
                    // This field isn't strictly necessary. If you don't include one then a generic update button will be generated for you.
                    array(
                        'type' => 'save',
                        'messages' => array(
                            'success' => __('Field Mapping is successfully updated.', 'engagebaygfaddon'),
                            'error' => __('Something went wrong. Please check the field mapping and try again.', 'engagebaygfaddon'),
                        ),
                    ),
                ),
            ),
        );
    }

    public function eb_gf_engagebay_fields()
    {
        $fields = array();
        $engagebay_fields = $this->eb_gf_get_engagebay_system_fields();
        $engagebay_fields = array_merge($engagebay_fields, $this->eb_gf_get_engagebay_custom_fields());

        foreach ($engagebay_fields as $engagebay_field) {
            $fields[] = array(
                'name' => $engagebay_field['name'],
                'label' => $engagebay_field['label'],
                'required' => $engagebay_field['is_required'],
                'type' => $engagebay_field['type'],
                'key_field' => array(),
                'engagebay_field_type' => $engagebay_field['field_type'] ?? '',
                'field_type' => array(
                    'name', 'radio', 'select', 'text', 'html', 'checkbox', 'hidden', 'textarea', 'number', 'date', 'email', 'phone', 'website', 'fileupload', 'time', 'multiselect', 'section', 'page', 'address', 'multicheckbox', 'SYSTEM_ADDRESS'
                ),
                'class' => 'x-small',
            );
        }

        return $fields;
    }

    private function eb_gf_get_engagebay_system_fields()
    {
        return array(
            array('name' => 'name', 'label' => __('First name', 'engagebaygfaddon'), 'is_required' => true, 'type' => 'SYSTEM'),
            array('name' => 'last_name', 'label' => __('Last name', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'company', 'label' => __('Company', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'email', 'label' => __('Email', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'phone', 'label' => __('Phone', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'country', 'label' => __('Country', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM'),
            array('name' => 'address', 'label' => __('Address', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM', 'field_type' => 'SYSTEM_ADDRESS'),
            array('name' => 'city', 'label' => __('City', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM', 'field_type' => 'SYSTEM_ADDRESS'),
            array('name' => 'state', 'label' => __('State', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM', 'field_type' => 'SYSTEM_ADDRESS'),
            array('name' => 'zip', 'label' => __('Zip', 'engagebaygfaddon'), 'is_required' => false, 'type' => 'SYSTEM', 'field_type' => 'SYSTEM_ADDRESS'),
        );
    }

    public function eb_gf_on_submit_form($entry, $form)
    {
        $engagebay_fields = $this->eb_gf_get_engagebay_fields();
        $form_entries = $this->eb_gf_get_form_entries($form, $entry);
        $contact_properties = $this->eb_gf_prepare_contact_properties($form_entries, $engagebay_fields);
        $email = $this->eb_gf_get_email_from_properties($contact_properties);

        if (empty($email)) {
            return;
        }

        $api_handler = new EngagebayGFAddonAPI();
        $engagebay_contact = $api_handler->search_contact($email);
        $tags = $this->eb_gf_prepare_tags($form);

        $payload = [
            'score' => 10,
            'properties' => $contact_properties,
            'tags' => $tags
        ];

        if ($engagebay_contact) {
            $payload['id'] = $engagebay_contact['id'];
            $api_handler->update_contact($payload);
        } else {
            $api_handler->add_contact($payload);
        }
    }

    private function eb_gf_get_engagebay_fields()
    {
        return array_merge($this->eb_gf_get_engagebay_system_fields(), $this->eb_gf_get_engagebay_custom_fields());
    }

    private function eb_gf_get_form_entries($form, $entry)
    {
        return $this->eb_gf_form_entries($form, $entry);
    }

    private function eb_gf_get_engagebay_custom_fields()
    {
        $engagebay_custom_fields = array();
        $api_handler = new EngagebayGFAddonAPI();
        $custom_fields = $api_handler->get_custom_fields();

        if ($custom_fields && is_array($custom_fields)) {
            foreach ($custom_fields as $custom_field) {
                $engagebay_custom_fields[] = array(
                    'name' => $custom_field['field_name'],
                    'label' => $custom_field['field_label'],
                    'field_type' => $custom_field['field_type'],
                    'is_required' => (bool)$custom_field['is_required'],
                    'type' => 'CUSTOM',
                );
            }
        }

        return $engagebay_custom_fields;
    }

    private function eb_gf_prepare_contact_properties($form_entries, $engagebay_fields)
    {
        $contact_properties = [];
        $fieldTypesMap = [
            'checkbox' => 'MULTICHECKBOX',
            'date' => 'DATE'
        ];

        foreach ($form_entries as $engagebay_field => $value) {
            $type = $this->eb_gf_get_field_type($engagebay_fields, $engagebay_field);
            if (strlen($type) === 0) {
                continue;
            }

            $value = explode("***", $value);

            $contact_properties[] = [
                'name' => $engagebay_field,
                'value' => $value[1],
                'field_type' => $fieldTypesMap[$value[0]] ?? 'TEXT',
                'is_searchable' => isset($fieldTypesMap[$value[0]]) && $value[0] == 'checkbox',
                'type' => $type
            ];

            if ($engagebay_field === "phone") {
                $contact_properties[count($contact_properties) - 1]['subtype'] = 'work';
            }
        }

        return $contact_properties;
    }

    private function eb_gf_get_email_from_properties($contact_properties)
    {
        foreach ($contact_properties as $property) {
            if ($property['name'] === 'email') {
                return $property['value'];
            }
        }
        return '';
    }

    private function eb_gf_prepare_tags($form)
    {
        $all_tags = $form['engagebay']['tag'];
        $split_tags = explode(",", $all_tags);
        return array_map(function ($tag) {
            return ['tag' => $tag];
        }, $split_tags);
    }

    private function eb_gf_get_field_type($fields, $name)
    {
        $filtered_fields = array_values(array_filter($fields, function ($field) use ($name) {
            return $field['name'] === $name;
        }));

        if (count($filtered_fields) > 0) {
            return $filtered_fields[0]['type'];
        }

        return '';
    }

    /**
     * @param $form
     * @param $entry
     * @return array
     */
    public function eb_gf_form_entries($form, $entry): array
    {
        $form_entries = [];
        foreach (EngageBayGFAddon::eb_gf_form_settings($form) as $key => $fieldId) {
            if (str_starts_with($key, 'fieldMap_') && !empty($fieldId)) {
                $input_type = $this->eb_gf_input_type($form['fields'], $fieldId);
                $key = substr($key, strlen('fieldMap_'));
                $value = rgar($entry, $fieldId);
                switch ($input_type) {
                    case 'checkbox':
                        $checkboxes = GFFormsModel::get_field($form, $form['engagebay']['fieldMap_' . $key]);
                        $checkbox_values = is_object($checkboxes) ? $checkboxes->get_value_export($entry) : '';
                        $checkbox_values = preg_replace('/,\s+/', ',', $checkbox_values);
                        $value = $checkbox_values;
                        break;
                    case 'date':
                        $value = strtotime( $value ) == '' ? '' : strtotime( $value );
                        break;

                }

                $form_entries[$key] = $input_type . '***' . $value;
            }
        }

        return $form_entries;
    }

    private function eb_gf_input_type($fields, $fieldId)
    {
        $filteredFields = array_values(array_filter($fields, function ($field) use ($fieldId) {
            return $field['id'] == $fieldId;
        }));

        if (count($filteredFields) > 0) {
            return $filteredFields[0]['type'];
        }

        return 'text';
    }
}